/***************************************************************************
 *   Copyright (C) 2005 by Nicolas PASCAL                                  *
 *   pascal@icare-pc15                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef HDF_EXCEPTIONS_H
#define HDF_EXCEPTIONS_H 
 
#include <string>
#include <iostream>
#include "mfhdf.h"
using namespace std;

/**
bad_hdf_file exception : the specified file couldn't be opened : it doesn't exist, or isn't an HDF one
*/

class bad_hdf_file {
public:
    bad_hdf_file(const string & _s) {
        s = _s;
    };
    ~bad_hdf_file() {}
    ;
    void what(void) const {
        if (&s != NULL)
            cerr<<"The file "<<s<<" can't be found or invalid HDF file"<<endl;
        else
            cerr<<"Empty file name"<<endl;
    };
private:
    string s;
};

/**
bad_hdf_file_opening_mode : the specified file couldn't be opened in the given mode.
*/

class bad_hdf_file_opening_mode {
public:
    bad_hdf_file_opening_mode(const string & _file, const string & _mode= string("r")) {
        file = _file;
        mode = _mode;
    };
    ~bad_hdf_file_opening_mode() {}
    ;
    void what(void) const {
        cerr<<"The file "<<file<<" can't be opened in mode "<<mode<<". Check the file's access rights"<<endl;
    };
private:
    string file;
    string mode;
};

class bad_hdf_compression {
public:
  bad_hdf_compression(const char *_file, const char *_sds, const int32 _code) {
    file = _file;
    sds = _sds;
    code = _code;
  }
  ~bad_hdf_compression() {}
  void what(void) const {
    char *str_code;

    switch(code) {
    case COMP_CODE_NONE :
      str_code = "COMP_CODE_NONE";
      break;
    case COMP_CODE_DEFLATE :
      str_code = "COMP_CODE_DEFLATE (gzip)";
      break;
    case COMP_CODE_SKPHUFF :
      str_code = "COMP_CODE_SKPHUFF (huffman)";
      break;
    default :
      str_code = "UNKNOWN";
      break;
    }
    cerr << file << ": failed to compress dataset " << sds << ", compression code: " << str_code << endl;
  }
private:
  string file;
  string sds;
  int32 code;
};

class bad_sd_file_info {
public:
    bad_sd_file_info(const char* _filename) {
        filename=_filename;
    };
    ~bad_sd_file_info() {}
    ;
    void what(void) const {
        cerr<<"Can't read the sd file info for "<<filename<<endl;
    };
private:
    string filename;
};

class bad_sd_attr_info {
public:
    bad_sd_attr_info(const char* _attr_name) {
        attr_name=_attr_name;
    };
    ~bad_sd_attr_info() {}
    ;
    void what(void) const {
        cerr<<"Can't read the sd attribute "<<attr_name<<endl;
    };
private:
    string attr_name;
};

class bad_sd_get_info {
public:
    bad_sd_get_info(const char* _sds_name) {
        sds_name=_sds_name;
    };
    ~bad_sd_get_info() {}
    ;
    void what(void) const {
        cerr<<"Can't get the info from the sds "<<sds_name<<endl;
    };
private:
    string sds_name;
};

class bad_sd_create {
public:
    bad_sd_create(const char* _filename) {
        filename=_filename;
    };
    ~bad_sd_create() {}
    ;
    void what(void) const {
        cerr<<"Can't create the sd for the file "<<filename<<endl;
    };
private:
    string filename;
};

class bad_sd_set_fill_value {
public:
    bad_sd_set_fill_value(const char* _filename, const void* _fill_value) {
        filename=_filename;
        fill_value=const_cast<void*>(_fill_value);
    };
    ~bad_sd_set_fill_value() {}
    ;
    void what(void) const {
        cerr<<"Can't set the sd fill value for the file "<<filename<<" to "<<fill_value<<endl;
    };
private:
    string filename;
    void* fill_value;
};

class bad_sd_set_fill_mode {
public:
    bad_sd_set_fill_mode(const char* _filename, const int32 _fill_mode) {
        filename=_filename;
        fill_mode=_fill_mode;
    };
    ~bad_sd_set_fill_mode() {}
    ;
    void what(void) const {
        cerr<<"Can't set the sd fill mode for the file "<<filename<<" to ";
        switch (fill_mode) {
            case SD_FILL: cerr<<"SD_FILL"<<endl;break;
            case SD_NOFILL: cerr<<"SD_NOFILL"<<endl;break;
            default: cerr<<"the unknown mode : "<<fill_mode<<". Must be SD_FILL or SD_NOFILL"<<endl;break;
        }
    };
private:
    string filename;
    int32 fill_mode;
};
class bad_sds_name {
public:
    bad_sds_name(const char* _filename, const char* _sds_name) {
        filename=_filename;
        sds_name=_sds_name;
    };
    ~bad_sds_name() {}
    ;
    void what(void) const {
        cerr<<"Can't find the sds "<<sds_name<<" in file "<<filename<<endl;
    };
private:
    string filename;
    string sds_name;
};
class bad_set_file_attribute {
public:
    bad_set_file_attribute(const char* _filename, const char* _attr_name) {
        filename=_filename;
        attr_name=_attr_name;
    };
    ~bad_set_file_attribute() {}
    ;
    void what(void) const {
        cerr<<"Can't set the attribute "<<attr_name<<" in file "<<filename<<endl;
    };
private:
    string filename;
    string attr_name;
};
class bad_set_sds_attribute {
public:
    bad_set_sds_attribute(const char* _filename, const char* _sds_name, const char* _attr_name) {
        filename=_filename;
        sds_name=_sds_name;
        attr_name=_attr_name;
    };
    ~bad_set_sds_attribute() {}
    ;
    void what(void) const {
        cerr<<"In file "<<filename<<" : can't set the attribute "<<attr_name<<" for the sds "<<sds_name<<endl;
    };
private:
    string filename;
    string sds_name;
    string attr_name;
};

class bad_sd_select {
public:
    bad_sd_select(const int32 _sds_index) {
        sds_index=_sds_index;
    };
    ~bad_sd_select() {}
    ;
    void what(void) const {
        cerr<<"Can't select the sds "<<sds_index<<endl;
    };
private:
    int32 sds_index;
};


class bad_sd_end_access {
public:
    bad_sd_end_access(void) {
    };
    ~bad_sd_end_access() {}
    ;
    void what(void) const {
        cerr<<"Can't end correctly the accessed sd"<<endl;
    };
};

class bad_sd_end {
public:
    bad_sd_end(void) {
    };
    ~bad_sd_end() {}
    ;
    void what(void) const {
        cerr<<"Can't close correctly the opened sd"<<endl;
    };
};

class sd_write_data_error {
    string filename;
    string sds_name;
    string error;
public:
    sd_write_data_error(const string & _filename, const string &_sds_name, const string & _error){
        filename=_filename;
        sds_name=_sds_name;
        error=_error;
    };
    
    ~sd_write_data_error() {}
    ;
    void what(void) const {
        cerr<<"sd_write_data_error while writing the sds "<<sds_name<<" in the file "<<filename<<" : "<<error<<endl;
    };
};

#endif
