/* Hdf_file.hpp */

#ifndef HDF_FILE_HPP
#define HDF_FILE_HPP

#include <iostream>
#include <vector>
#include "hdf.h"
#include "mfhdf.h"
#include "Hdf_attr.hpp"
#include "Hdf_sds.hpp"
#include "Hdf_common.hpp"
#include "Hdf_exceptions.hpp"

#define SDS_MAX_RANK 32
#define SDS_NAME_MAX_LEN 256
#define DEFAULT_COMP_LEVEL 4

class Hdf_file {
  std::string filename;
  int32 sd_id;
  int32 n_sds;
  int32 n_file_attrs;
  Hdf_attr *file_attrs;
  Hdf_sds *sds;
  void Hdf_file_init(const char *filename, int32 access);

public:
  typedef Hdf_sds::sds_infos_type sds_infos_type;
  Hdf_file(const char *filename, int32 access, std::vector<sds_infos_type> sds_infos = std::vector<sds_infos_type>(), int32 fill_mode = SD_FILL, VOIDP fill_value=NULL, int32 compression_type = COMP_CODE_NONE, int32 compression_level = DEFAULT_COMP_LEVEL);
  Hdf_file::Hdf_file(const char *filename, int32 access, Hdf_file::sds_infos_type *sds_infos, const size_t nsds, int32 fill_mode = SD_FILL, VOIDP fill_value=NULL, int32 compression_type = COMP_CODE_NONE, int32 compression_level = DEFAULT_COMP_LEVEL);
  Hdf_file(const char *filename, int32 access, int32 n_sds_array, Hdf_sds *sds_array, int32 fill_mode = SD_FILL, VOIDP fill_value=NULL, int32 compression_type = COMP_CODE_NONE, int32 compression_level = DEFAULT_COMP_LEVEL);
  Hdf_file(const Hdf_file &hdf_file);
  ~Hdf_file();
  std::string get_filename() const { return filename; }
  int32 get_sd_id() const { return sd_id; }
  int32 get_n_sds() const { return n_sds; }
  int32 get_n_file_attrs() const { return n_file_attrs; }
  Hdf_attr get_attr(const char *file_attr) const;
  Hdf_sds get_sds(const char *sds_name) const;
  Hdf_sds get_sds(const int32 &sds_index) const;
  std::string get_sds_name( const int32 &sds_index );
  const int32 get_sds_index(const char *sds_name);
  void get_sds_array(Hdf_sds *sds_array);
  void* get_fill_value(void);
  void set_sds(int32 sds_index, Hdf_sds sds);
  Hdf_sds read_sds(const char *sds_name, void *sds_data = NULL, int32 *start = NULL, int32 *stride = NULL, int32 *edges = NULL);
  Hdf_sds read_sds(const int32 sds_index, void *sds_data = NULL, int32 *start = NULL, int32 *stride = NULL, int32 *edges = NULL);
  void write_sds(const char *sds_name, const void *sds_data, int32 *start, int32 *stride, int32 *edges, float64 cal = 0., float64 offset = 0., float64 cal_err = 0., float64 off_err = 0.);
  void write_sds(const int32 sds_index, const void *sds_data, int32 *start, int32 *stride, int32 *edges, float64 cal = 0., float64 offset = 0., float64 cal_err = 0., float64 off_err = 0.);
  std::string list_of_attributes(const int32 chars_to_display = 80, const int32 numbers_to_display = 10) const;
  std::string list_of_sds() const;
  void set_file_attr(const char *attr_name, const int val_type, const int32 val_length, VOIDP attr_val );
  void set_sds_attr(const int32 sds_index, const char *attr_name, const int val_type, const int32 val_length, VOIDP attr_val );
  void set_sds_attr(const char* sds_name, const char *attr_name, const int val_type, const int32 val_length, VOIDP attr_val );
  
  Hdf_file &operator= (const Hdf_file &hdf_file);
  friend std::ostream &operator<< (std::ostream &stream, const Hdf_file &hdf_file);
  
};

#endif
