/* allocation.hpp */

#ifndef ALLOCATION_HPP
#define ALLOCATION_HPP

#include <new>
#include <cassert>

#undef ALLOCATION_DEBUG

#ifdef ALLOCATION_DEBUG
#define PDEBUG { std::fprintf(stderr, "call to %s (%s)\n", __PRETTY_FUNCTION__, __FILE__); }
#else
#define PDEBUG
#endif

template <typename data_type>
void allocate(data_type ** &data, int nrows, int ncols) {

  PDEBUG;

  data = new(std::nothrow) (data_type *)[nrows] ; assert(data != NULL);
  data[0] = new(std::nothrow) data_type[nrows*ncols]; assert(data[0] != NULL);
  
  for (int irow = 1 ; irow < nrows ; irow++) {
    data[irow] = data[0] + irow*ncols;
  }
}

template <typename data_type>
void deallocate(data_type ** &data) {

  PDEBUG;

  delete data[0];
  delete data;

}

#endif
