/* read_any.h */

#ifndef READ_ANY_H
#define READ_ANY_H

#include <hdf/mfhdf.h> /* int32 hdf type */
#include "sds_props.h"
#include "xrit/include/libxrit.h"

/** \brief a wrapper to functions <i>read_fis</i> and <i>read_xrit</i>;
 * looks <i>filename</i>'s extension and calls the appropriate function.
 *
 * @param filename name of the file to read
 * @param ppBuffer the address to a pointer to the buffer where data will be written to (the buffer is allocated by <i>read_any</i> and must be freed by the caller)
 * @param pNpix the address to a variable npix, which will be filled with the number of pixels read from <b>filename</b>
 * @param pData_type the address to a variable data_type, will will be filled with a data type code (DFNT_UINT8 or DFNT_UINT16)
 * @param rows_infos the address to a structure <b>xrit_rows_infos_t</b>, containing infos about rows validity and time (allocated by <i>read_any</i>)
 * @param number_of_sds size of the <b>sds</b> array
 * @param sds array of datasets, may be updated by <i>read_any</i> (calibration scale and offset fields)
 * @param log may be set to a non_null value to send error messages to stderr
 * @return 0 if no error occured, a non-null value otherwise
 */
int read_any(const char *filename, void **ppBuffer, size_t *pNpix, int32 *pData_type, xrit_rows_infos_t *rows_infos, int number_of_sds, sds_props_t *sds, const unsigned char log);

#endif
