/*
  cc -O3 sds2ppm.c -o sds2ppm -I$HDFINC -L$HDFLIB -lmfhdf -ldf -lz -ljpeg -lm
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "mfhdf.h"
#include <float.h>      /* Defines FLT_MAX and DBL_MAX RedHat 7.2 */

#define MAXNAMELENGTH 200
#define BGCOLOR	{35,35,117}
#define	MAXVALUES	USHRT_MAX
#define FILLVALUE	-1

typedef struct {
  char name[MAXNAMELENGTH];
  int32 id, index, num_type, rank, n_attr, Nl, Np, *plane, Nplanes;
  int32 start[MAX_VAR_DIMS], edges[MAX_VAR_DIMS], dim_sizes[MAX_VAR_DIMS];
  void *data, *fillvalue;
  char negate;
} SDS;

enum {RED, GREEN, BLUE, INT};



int main(int argc, char *argv[])
{
char filename1[3][MAXNAMELENGTH], filename2[MAXNAMELENGTH];
SDS sds[3];
int32 sd_id[3];
int ib, ipt, k, j, idx, Nl=0, Np=0, npix;
int nrows=-1, ncols=-1, num_type=-1;
FILE *fd, *fdin[3];
unsigned char *RGB, *ptr;
unsigned char list[4][100];
int bgRGB[3]=BGCOLOR;
int Npoints[4], *sdsval[4], *colidx[4];
unsigned char optionname[4][15], option[4][9];
float val1, val2, userfill=FLT_MAX;
unsigned char alldone, colorize=FALSE, verbose=FALSE, filter=TRUE, nocyan=FALSE;
float sdsvalto8bit[4][MAXVALUES];
int sdsvalidx;
double hue, sat, intensity, newintensity;
float red, green, blue;
char string[50];

  strcpy(option[RED], "-R=");
  strcpy(option[GREEN], "-G=");
  strcpy(option[BLUE], "-B=");
  strcpy(option[INT], "-int=");
  strcpy(optionname[RED], "red band");
  strcpy(optionname[GREEN], "green band");
  strcpy(optionname[BLUE], "blue band");
  strcpy(optionname[INT], "intensity");
  for (ib=0; ib<3; ib++)
    filename1[ib][0] = (char)NULL;
  filename2[0] = (char)NULL;

  sds[RED].index = 0;
  sds[GREEN].index = 1;
  sds[BLUE].index = 2;
  strcpy(list[RED],   "0,5120");
  strcpy(list[GREEN], "0,5120");
  strcpy(list[BLUE],  "0,5120");
  list[INT][0] =  (char)NULL;
  sds[RED].name[0] = (char)NULL;
  sds[GREEN].name[0] = (char)NULL;
  sds[BLUE].name[0] = (char)NULL;
  sds[RED].negate = FALSE;
  sds[GREEN].negate = FALSE;
  sds[BLUE].negate = FALSE;

  ib=0;

  for (j=1; j<argc; j++)
    if (strcmp(argv[j], "-v") == 0) {
      fprintf(stderr, "Verbose mode requested\n");
      verbose = TRUE;
    }

  for (j=1; j<argc; j++) {
    if (strstr(argv[j], "-of=") == argv[j] ) {
        if ( sscanf(argv[j], "-of=%s", filename2) != 1 ) {
          fprintf(stderr, "Cannot parse output file\n");
          exit(-1);
        }
        if (verbose)
          fprintf(stderr, "Output file: %s\n", filename2);
    } else if (strstr(argv[j], option[RED]) == argv[j] ) {
        strcpy(sds[RED].name, argv[j] + strlen(option[RED]));
        if ( ! sds[RED].name[0] ) {
          fprintf(stderr, "Cannot parse %s request\n", optionname[RED]);
          exit(-1);
        }
        if ( (ptr = strchr(sds[RED].name, 44)) ) {
          strcpy(list[RED], ptr + 1);
          *ptr = (char)NULL;
        }
        if ( sscanf(sds[RED].name, "%d", &sds[RED].index) != 1 ) {
            if (verbose)
              fprintf(stderr, "SDS selected for %s: %s %s\n", optionname[RED], sds[RED].name, list[RED]);
        } else {
            if (sds[RED].index < 0) {
              sds[RED].index = - sds[RED].index;
              sds[RED].negate = TRUE;
            }
            sds[RED].name[0] = (char)NULL;
            if (verbose)
              fprintf(stderr, "Index of %s: %d %s\n", optionname[RED], sds[RED].index, list[RED]);
            sds[RED].index--;
        }
        sds[RED].dim_sizes[0] = nrows;	/* In case band is a flat binary file, not HDF */
        sds[RED].dim_sizes[1] = ncols;
        sds[RED].num_type = num_type;
    } else if (strstr(argv[j], option[GREEN]) == argv[j] ) {
        strcpy(sds[GREEN].name, argv[j] + strlen(option[GREEN]));
        if ( ! sds[GREEN].name[0] ) {
          fprintf(stderr, "Cannot parse %s request\n", optionname[GREEN]);
          exit(-1);
        }
        if ( (ptr = strchr(sds[GREEN].name, 44)) ) {
          strcpy(list[GREEN], ptr + 1);
          *ptr = (char)NULL;
        }
        if ( sscanf(sds[GREEN].name, "%d", &sds[GREEN].index) != 1 ) {
            if (verbose)
              fprintf(stderr, "SDS selected for %s: %s %s\n", optionname[GREEN], sds[GREEN].name, list[GREEN]);
        } else {
            if (sds[GREEN].index < 0) {
              sds[GREEN].index = - sds[GREEN].index;
              sds[GREEN].negate = TRUE;
            }
            sds[GREEN].name[0] = (char)NULL;
            if (verbose)
              fprintf(stderr, "Index of %s: %d %s\n", optionname[GREEN], sds[GREEN].index, list[GREEN]);
            sds[GREEN].index--;
        }
        sds[GREEN].dim_sizes[0] = nrows;	/* In case band is a flat binary file, not HDF */
        sds[GREEN].dim_sizes[1] = ncols;
        sds[GREEN].num_type = num_type;
    } else if (strstr(argv[j], option[BLUE]) == argv[j] ) {
        strcpy(sds[BLUE].name, argv[j] + strlen(option[BLUE]));
        if ( ! sds[BLUE].name[0] ) {
          fprintf(stderr, "Cannot parse %s request\n", optionname[BLUE]);
          exit(-1);
        }
        if ( (ptr = strchr(sds[BLUE].name, 44)) ) {
          strcpy(list[BLUE], ptr + 1);
          *ptr = (char)NULL;
        }
        if ( sscanf(sds[BLUE].name, "%d", &sds[BLUE].index) != 1 ) {
            if (verbose)
              fprintf(stderr, "SDS selected for %s: %s %s\n", optionname[BLUE], sds[BLUE].name, list[BLUE]);
        } else {
            if (sds[BLUE].index < 0) {
              sds[BLUE].index = - sds[BLUE].index;
              sds[BLUE].negate = TRUE;
            }
            sds[BLUE].name[0] = (char)NULL;
            if (verbose)
              fprintf(stderr, "Index of %s: %d %s\n", optionname[BLUE], sds[BLUE].index, list[BLUE]);
            sds[BLUE].index--;
        }
        sds[BLUE].dim_sizes[0] = nrows;	/* In case band is a flat binary file, not HDF */
        sds[BLUE].dim_sizes[1] = ncols;
        sds[BLUE].num_type = num_type;
    } else if (strstr(argv[j], option[INT]) == argv[j] ) {
        strcpy(list[INT], argv[j] + strlen(option[INT]));
        if ( list[INT][0] == (char)NULL ) {
          fprintf(stderr, "Cannot parse %s request\n", optionname[INT]);
          exit(-1);
        }
    } else if (strstr(argv[j], "-type=") == argv[j] ) {
        strcpy(string, argv[j] + strlen("-type="));
        if ( ! string[0] ) {
          fprintf(stderr, "Cannot parse data type (%s)\n", argv[j]);
          exit(-1);
        }
        if (strcmp(string, "int16") == 0)
          num_type = DFNT_INT16;
        else if (strcmp(string, "uint16") == 0)
          num_type = DFNT_UINT16;
        else {
          fprintf(stderr, "Data type not supported (%s)\n", string);
          exit(-1);
        }
        if (verbose)
          fprintf(stderr, "Data type: %s\n", string);
    } else if (strstr(argv[j], "-dim=") == argv[j] ) {
        if (sscanf(argv[j], "-dim=%dx%d", &ncols, &nrows) != 2) {
          fprintf(stderr, "Cannot parse dimensions (%s)\n", argv[j]);
          exit(-1);
        }
        if (verbose)
          fprintf(stderr, "Dimensions: %dx%d\n", ncols, nrows);
    } else if (strstr(argv[j], "-bg=") == argv[j] ) {
        if (sscanf(argv[j], "-bg=%d,%d,%d", &bgRGB[0], &bgRGB[1], &bgRGB[2]) != 3) {
          fprintf(stderr, "Cannot parse background color\n");
          exit(-1);
        }
        if (verbose)
          fprintf(stderr, "Background color: %d %d %d\n", bgRGB[0], bgRGB[1], bgRGB[2]);
    } else if (strstr(argv[j], "-fill=") == argv[j] ) {
        if (sscanf(argv[j], "-fill=%g", &userfill) != 1) {
          fprintf(stderr, "Cannot parse user fill value\n");
          exit(-1);
        }
        if (verbose)
          fprintf(stderr, "User fill value: %g\n", userfill);
    } else if (strcmp(argv[j], "-nofilter") == 0) {
        if (verbose)
          fprintf(stderr, "Unnatural colored pixels not filtered\n");
        filter = FALSE;
    } else if (strcmp(argv[j], "-nocyan") == 0) {
        if (verbose)
          fprintf(stderr, "Cyan color will be filtered and replaced by gray\n");
        nocyan = TRUE;
    } else if (strcmp(argv[j], "-colorize") == 0) {
        if (verbose)
          fprintf(stderr, "Colorize fill values\n");
        colorize = TRUE;
    } else if ( filename2[0] == (char)NULL  &&
                strcmp(argv[j], "-") == 0 ) {
        strcpy(filename2, "stdout");
        if (verbose)
          fprintf(stderr, "Output file: %s\n", filename2);
    } else if (strcmp(argv[j], "-v") != 0) {		/* Ignore -v option (already parsed) */
        strcpy(filename1[ib], argv[j]);
        if (verbose)
          fprintf(stderr, "Input file: %s\n", filename1[ib]);
        ib++;
    }
  }

  if ( filename1[0][0] == (char)NULL  ||
       filename2[0] == (char)NULL  ||
       ( sds[RED].index < 0    &&  ! sds[RED].name[0] )  ||
       ( sds[GREEN].index < 0  &&  ! sds[GREEN].name[0] )  ||
       ( sds[BLUE].index < 0   &&  ! sds[BLUE].name[0] ) ) {
    fprintf(stderr, "Unable to get all arguments\n");
    exit(-1);
  }

  if ( filename1[1][0] == (char)NULL )
    strcpy(filename1[1], filename1[0]);
  if ( filename1[2][0] == (char)NULL )
    strcpy(filename1[2], filename1[0]);

  for (ib=0; ib<3; ib++) {
    if ( (sd_id[ib] = SDstart(filename1[ib], DFACC_READ)) == -1  &&
         (fdin[ib] = fopen(filename1[ib], "r")) == NULL ) {
      fprintf(stderr, "Cannot open file %s.\n", filename1[ib]);
      exit(-1);
    }
    sds[ib].fillvalue = NULL;
    if (sd_id[ib] != -1) {
      if ( sds[ib].name[0]  &&
           (sds[ib].index = SDnametoindex(sd_id[ib], sds[ib].name)) == -1 ) {
        fprintf(stderr, "Cannot find SDS %s\n", sds[ib].name);
        SDend(sd_id[ib]);
        exit(-1);
      }
      if ( (sds[ib].id = SDselect(sd_id[ib], sds[ib].index)) == -1 ) {
        fprintf(stderr, "Cannot select SDS no. %d\n", sds[ib].index);
        SDend(sd_id[ib]);
        exit(-1);
      }
      if (SDgetinfo(sds[ib].id, sds[ib].name, &sds[ib].rank, sds[ib].dim_sizes, &sds[ib].num_type, &sds[ib].n_attr) == -1) {
        fprintf(stderr, "Can't get info from SDS \"%s\" in file %s.\n", sds[ib].name, filename1[ib]);
        SDendaccess(sds[ib].id);
        SDend(sd_id[ib]);
        exit(-1);
      }
      sds[ib].fillvalue = (void *) malloc(1 * DFKNTsize(sds[ib].num_type));
      if (SDgetfillvalue(sds[ib].id, sds[ib].fillvalue) != -1) {
          if (verbose) {
            fprintf(stderr, "Fillvalue: ");
            switch(sds[ib].num_type) {
              case DFNT_INT16:  fprintf(stderr, "%d\n", *( int16*)sds[ib].fillvalue);  break;
              case DFNT_UINT16: fprintf(stderr, "%d\n", *(uint16*)sds[ib].fillvalue);  break;
            }
          }
      } else {
          if (verbose)
            fprintf(stderr, "Can't get fill value from SDS \"%s\" in file %s.\n", sds[ib].name, filename1[ib]);
          free(sds[ib].fillvalue);
        sds[ib].fillvalue = NULL;
        }
sds[ib].fillvalue = NULL;
    }
    if ( sds[ib].dim_sizes[0] > Nl  ||
         sds[ib].dim_sizes[1] > Np ) {
      Nl = sds[ib].dim_sizes[0];
      Np = sds[ib].dim_sizes[1];
    }
    if ( sds[ib].dim_sizes[0] / (float)Nl != sds[ib].dim_sizes[1] / (float)Np ) {
      fprintf(stderr, "Dimensions of SDSs are different\n");
      SDendaccess(sds[ib].id);
      SDend(sd_id[ib]);
      exit(-1);
    }
    if ( sds[ib].num_type != sds[0].num_type ) {
      if (verbose) fprintf(stderr, "Data types of SDSs are different\n");
/*
      SDendaccess(sds[ib].id);
      SDend(sd_id[ib]);
      exit(-1);
*/
    }
    sds[ib].data = (void *) malloc(sds[ib].dim_sizes[1] * DFKNTsize(sds[ib].num_type));
  }

  for (ib=0; ib<4; ib++) {
    if (ib == INT  &&  ! list[ib][0]) continue;
    Npoints[ib] = 1;
    while ( ptr = strchr(list[ib], 44) ) {
      *ptr = 32;
      Npoints[ib]++;
    }
    if ( Npoints[ib] % 2 == 1 ) {
      fprintf(stderr, "Too few arguments in %s specification\n", optionname[ib]);
      exit(-1);
    }
    Npoints[ib] /= 2;
    if (verbose)
      fprintf(stderr, "%d point(s):\n", Npoints[ib]);
    sdsval[ib] = (int *) malloc( MAX(Npoints[ib], 2) * sizeof(int));
    colidx[ib] = (int *) malloc( MAX(Npoints[ib], 2) * sizeof(int));
    ptr = list[ib];
    for (k=0; k<Npoints[ib]; k++) {
      if ( sscanf(ptr, "%d %d", &sdsval[ib][k], &colidx[ib][k]) != 2 ) {
        fprintf(stderr, "Cannot parse arguments in %s specification\n", optionname[ib]);
        exit(-1);
      }
      if ( ptr = strchr(ptr, 32) ) ptr++;
      if ( ptr = strchr(ptr, 32) ) ptr++;
    }
    if (Npoints[ib] == 1) {
      Npoints[ib] = 2;
      sdsval[ib][1] = colidx[ib][0];
      colidx[ib][0] = 0;
      colidx[ib][1] = 255;
    } 
    if (verbose)
      for (k=0; k<Npoints[ib]; k++)
        fprintf(stderr, "  %d %d\n", sdsval[ib][k], colidx[ib][k]);
  }
  if (verbose) {
    fprintf(stderr, "Stretch option:");
    for (ib=0; ib<4; ib++) {
      if (ib == INT  &&  ! list[ib][0]) continue;
      fprintf(stderr, " %s%d", option[ib], sds[ib].index + 1);
      for (k=0; k<Npoints[ib]; k++)
        fprintf(stderr, ",%d,%d", sdsval[ib][k], colidx[ib][k]);
    }
    fprintf(stderr, "\n");
    fprintf(stderr, "Dimensions: %dx%d\n", Np, Nl);
  }

  RGB = (char *) malloc(3 * Np * sizeof(char));

  for (ib=0; ib<4; ib++) {
    if (ib == INT  &&  list[INT][0] == (char)NULL ) continue;
    for (k=0; k<MAXVALUES; k++) {
      if (ib == INT)
        val1 = k * 255. / MAXVALUES;
      else
      switch (sds[ib].num_type) {
        case DFNT_INT16:
          val1 = k - (int)(MAXVALUES / 2) + 0.5;
          break;
        case DFNT_UINT16:
          val1 = k + 0.5;
          break;
      }
      sdsvalto8bit[ib][k] = FILLVALUE;
      if (val1 <= sdsval[ib][0])
        sdsvalto8bit[ib][k] = colidx[ib][0];
      else if (val1 >= sdsval[ib][Npoints[ib] - 1])
        sdsvalto8bit[ib][k] = colidx[ib][Npoints[ib] - 1];
      else
        for (ipt=0; ipt<Npoints[ib] - 1; ipt++) {
          if ( val1 >= sdsval[ib][ipt]  &&
               val1 <= sdsval[ib][ipt + 1] ) {
            val2 = colidx[ib][ipt] + (val1 - sdsval[ib][ipt]) * (colidx[ib][ipt + 1] - colidx[ib][ipt]) / (sdsval[ib][ipt + 1] - sdsval[ib][ipt]);
            if (val2 < 0) val2 = 0;
            if (val2 > 255) val2 = 255;
            if (ib != INT  &&  sds[ib].negate)
              sdsvalto8bit[ib][k] = 255 - val2;
            else
              sdsvalto8bit[ib][k] = val2;
            break;
          }
        }
    }
  }
  for (k=0; k<MAXVALUES; k++) {
    if ( sdsvalto8bit[0][k] < 0  ||
         sdsvalto8bit[1][k] < 0  ||
         sdsvalto8bit[2][k] < 0 ) {
        sdsvalto8bit[0][k] = bgRGB[0];
        sdsvalto8bit[1][k] = bgRGB[1];
        sdsvalto8bit[2][k] = bgRGB[2];
    } else if (list[INT][0] != (char)NULL) {
        red = sdsvalto8bit[0][k] / 255;
        green = sdsvalto8bit[1][k] / 255;
        blue = sdsvalto8bit[2][k] / 255;
        intensity = (red + green + blue) / 3;
        newintensity = sdsvalto8bit[INT][(int)(intensity * MAXVALUES + 0.5)] / 255;
        if (intensity != 0  &&  intensity < 1) {
          sdsvalto8bit[0][k] = sdsvalto8bit[0][k] * newintensity / intensity;
          sdsvalto8bit[1][k] = sdsvalto8bit[1][k] * newintensity / intensity;
          sdsvalto8bit[2][k] = sdsvalto8bit[2][k] * newintensity / intensity;
          if ( sdsvalto8bit[0][k] > 255  ||
               sdsvalto8bit[1][k] > 255  ||
               sdsvalto8bit[2][k] > 255 ) {
            sdsvalto8bit[0][k] = bgRGB[0];
            sdsvalto8bit[1][k] = bgRGB[1];
            sdsvalto8bit[2][k] = bgRGB[2];
          }
        }
    }
  }
 
  if (strcmp(filename2, "stdout") == 0)
    fd = stdout;
  else if ( (fd = fopen(filename2, "w")) == NULL ) {
    fprintf(stderr, "Cannot open file %s.\n", filename2);
    exit(-1);
  }
  fprintf(fd, "P6\n%d %d\n255\n", Np, Nl);
  for (k=0; k<Nl; k++) {
    for (ib=0; ib<3; ib++) {
      if (k % (Nl / sds[ib].dim_sizes[0]) != 0)
        continue;
      if (sd_id[ib] != -1) {
          sds[ib].start[0] = k * sds[ib].dim_sizes[0] / (float)Nl;
          sds[ib].start[1] = 0;
          sds[ib].edges[0] = 1;
          sds[ib].edges[1] = sds[ib].dim_sizes[1];
          if (SDreaddata(sds[ib].id, sds[ib].start, NULL, sds[ib].edges, sds[ib].data) == -1) {
            fprintf(stderr, "  Can't read SDS \"%s\" in file %s. File ignored.\n\n", sds[ib].name, filename1[ib]);
            SDendaccess(sds[ib].id);
            SDend(sd_id[ib]);
            exit(-1);
          }
      } else {
          if ( fread(sds[ib].data, DFKNTsize(sds[ib].num_type), sds[ib].dim_sizes[1], fdin[ib]) != sds[ib].dim_sizes[1] ) {
            fprintf(stderr, "  Can't read file %s. File ignored.\n\n", filename1[ib]);
            fclose(fdin[ib]);
            exit(-1);
          }
      }
    }
    for (j=0; j<Np; j++) {
      for (ib=0; ib<3; ib++) {
        RGB[3 * j + ib] = bgRGB[ib];
        val1 = FLT_MAX;
        idx = j * sds[ib].dim_sizes[1] / (float)Np;
        alldone = FALSE;
        switch (sds[ib].num_type) {
          case DFNT_INT16:
            val1 = ((int16 *)sds[ib].data)[idx];
if (val1==-200) val1=20000;
/* Colorize coastlines and other fill values */
            if (colorize) {
              if (val1 == -200) {	/* fill value */
                  RGB[3 * j]     = bgRGB[0];
                  RGB[3 * j + 1] = bgRGB[1];
                  RGB[3 * j + 2] = bgRGB[2];
                  alldone = TRUE;
              } else if (val1 == -220) {	/* graticule */
                  RGB[3 * j]     = 0;
                  RGB[3 * j + 1] = 0;
                  RGB[3 * j + 2] = 0;
                  alldone = TRUE;
              } else if (val1 == -250) {	/* outside valid domain */
                  RGB[3 * j]     = 255;
                  RGB[3 * j + 1] = 255;
                  RGB[3 * j + 2] = 255;
                  alldone = TRUE;
              } else if (val1 == -300) {	/* coastlines */
                  RGB[3 * j]     = 255;
                  RGB[3 * j + 1] = 255;
                  RGB[3 * j + 2] = 0;
                  alldone = TRUE;
              }
            }
            if ( sds[ib].fillvalue  &&
                 ((int16 *)sds[ib].data)[idx] == *((int16 *)sds[ib].fillvalue) )
              val1 = FLT_MAX;
            sdsvalidx = (int)(val1 + (int)(MAXVALUES / 2) + 0.5);
            break;
          case DFNT_UINT16:
            val1 = ((uint16 *)sds[ib].data)[idx];
            if ( sds[ib].fillvalue  &&
                 ((uint16 *)sds[ib].data)[idx] == *((uint16 *)sds[ib].fillvalue) )
              val1 = FLT_MAX;
            sdsvalidx = (int)(val1 + 0.5);
            break;
          default:
            fprintf(stderr, "Data type not supported\n");
            exit(-1);
        }
        if (alldone)
          break;
        if  (val1 == FLT_MAX  ||
             val1 == userfill ) {
          RGB[3 * j] = RGB[3 * j + 1] = RGB[3 * j + 2] = bgRGB[ib];
          alldone = TRUE;
          break;
        }
        RGB[3 * j + ib] = (unsigned char)sdsvalto8bit[ib][sdsvalidx];
      }

      if (filter) {

/* Filters Magenta out
        if ( RGB[3 * j] > 240  &&
             RGB[3 * j + 1] == 0  &&
             RGB[3 * j + 2] > 240 )
          RGB[3 * j] = RGB[3 * j + 1] = RGB[3 * j + 2] = 255;
*/
/* Filters Cyan out */
        if ( RGB[3 * j] == 0  &&
             RGB[3 * j + 1] > 240  &&
             RGB[3 * j + 2] > 240 )
          RGB[3 * j] = RGB[3 * j + 1] = RGB[3 * j + 2] = 255;
        if ( nocyan  &&
             RGB[3 * j + 1] == 255  &&
             RGB[3 * j + 2] == 255 )
          RGB[3 * j + 1] = RGB[3 * j + 2] = RGB[3 * j];
/* Filters Red out */
        if ( RGB[3 * j] > 240  &&
             RGB[3 * j + 1] < 20  &&
             RGB[3 * j + 2] < 20 )
          RGB[3 * j] = RGB[3 * j + 1] = RGB[3 * j + 2] = 255;
/* Filters Yellow out */
        if ( ! colorize &&
             RGB[3 * j] > 240  &&
             RGB[3 * j + 1] > 240  &&
             RGB[3 * j + 2] < 20 )
          RGB[3 * j] = RGB[3 * j + 1] = RGB[3 * j + 2] = 255;
/* Filters Green out */
        if ( RGB[3 * j] == 0  &&
             RGB[3 * j + 1] > 240  &&
             RGB[3 * j + 2] == 0 )
          RGB[3 * j] = RGB[3 * j + 1] = RGB[3 * j + 2] = 255;
/* Colorize coastlines */
/*
        if ( RGB[3 * j] == RGB[3 * j + 1]  &&
             RGB[3 * j] == RGB[3 * j + 2]  &&
             RGB[3 * j] == 0 ) {
          RGB[3 * j] = 255;
          RGB[3 * j + 1] = 255;
          RGB[3 * j + 2] = 0;
        }
*/
      }

    }

    if ( (npix = fwrite(RGB, 3 * sizeof(char), Np, fd)) != Np ) {
      fprintf(stderr, "Cannot write row %d in file %s (%d pixels written)\n", k, filename2, npix);
      exit(-1);
    }

  }

  fclose(fd);
  for (ib=0; ib<3; ib++) {
    if (sd_id[ib] == -1)
      fclose(fdin[ib]);
    else {
      SDendaccess(sds[ib].id);
      SDend(sd_id[ib]);
    }
    free(sds[ib].data);
    free(sdsval[ib]);
/*
    free(colidx[ib]);
*/
  }
  free(RGB);

  return 0;

}
