/* seviri_latlon.c */

#include <stdio.h>
#include <stdlib.h>
#include "msg_navigation.h"

static float latitudes[3712*3712];
static float longitudes[3712*3712];

void seviri_compute_latlon(float *latitudes, float *longitudes) {
  const int nrows = 3712;
  const int ncols = 3712;
  const int coff  = COFF;
  const int loff  = LOFF;
  const double NaN = 0./0.;
  int irow;
  int icol;
  double latitude;
  double longitude;

  for (irow = 0 ; irow < nrows ; irow++) {
    for (icol = 0 ; icol < ncols ; icol++) {
      pixcoord2geocoord(icol + 1, irow + 1, coff, loff, &latitude, &longitude);
      if (latitude < -90. || latitude > 90.) latitude = NaN;
      if (longitude < -180. || longitude > 180.) longitude = NaN;
      latitudes[(nrows - irow - 1)*ncols+(ncols - icol - 1)] = (float) latitude;
      longitudes[(nrows - irow - 1)*ncols+(ncols - icol - 1)] = (float) longitude;
    }
  }
}

int main(void) {

  const char file_latitudes[] = "seviri_latitudes.raw";
  const char file_longitudes[] = "seviri_longitudes.raw";
  FILE *stream;

  seviri_compute_latlon(latitudes, longitudes);

  stream = fopen(file_latitudes, "w");
  if (stream == NULL) {
    perror(file_latitudes);
    exit(EXIT_FAILURE);
  }

  fwrite(latitudes, 3712*3712*sizeof(float), 1, stream);

  if (fclose(stream) != 0) {
    perror(file_latitudes);
    exit(EXIT_FAILURE);
  }

  stream = fopen(file_longitudes, "w");
  if (stream == NULL) {
    perror(file_longitudes);
    exit(EXIT_FAILURE);
  }

  fwrite(longitudes, 3712*3712*sizeof(float), 1, stream);

  if (fclose(stream) != 0) {
    perror(file_longitudes);
    exit(EXIT_FAILURE);
  }
  
}
