/* track.h */

#ifndef _TRACK_H
#define _TRACK_H

#include <math.h>

#ifndef Tearth
#warning "Tearth not defined: define it as Tearth = 24*60*60 (in seconds)"
#define Tearth 24*60*60
#endif

/** returns 1 if the latitude is valid for a given orbital inclination (inclination and latitude in radians) 
 *  returns 0 otherwise
 */
#define track_valid_latitude(latitude, incli) (fabs(sin(latitude)) < fabs(sin(incli))) 

/** returns the longitude for a given latitude, with Tsat, incli and node_longitude fixed (orbital parameters) 
 * Tsat in minutes (unless Tearth is redefined), latitude, incli and node_longitude in radians
 * longitude is given in radians
 */
#define track_longitude(latitude, Tsat, incli, node_longitude) ( atan2(sin(latitude)/tan(incli), \
           cos(asin(sin(latitude)/sin(incli)))) - (Tsat)/(Tearth) * asin(sin(latitude)/sin(incli)) + (node_longitude) )

/* returns track time for a given latitude, with Tsat and asc_node_time (or desc_node_time) fixed
 *
 * latitude in radians, Tsat and asc_node_time in what you like
 * (I use seconds for Tsat and seconds since Unix Epoch for asc_node_time)
 *
 * NB. We only suppose here a circular orbit. We take neither account of Keplerian law of areas nor of
 * perigee advance
 * 
 */
#define track_time_asc(latitude, Tsat, asc_node_time) ( asc_node_time + latitude * Tsat / (2*M_PI))
#define track_time_desc(latitude, Tsat, desc_node_time) ( desc_node_time - latitude * Tsat / (2*M_PI))

#endif
