/* read_seviri_prologue.c */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <time.h>
#include <stddef.h>
#include <assert.h>
#include "read_xrit.h"
#include "xrit_swap.h"

xrit_rows_infos_t rows_infos;

int main(int argc, char *argv[]) {

  char *input_file;
  int nsegments;
  xrit_header_t *xrit_header;
  MSG_Prologue prologue;
  double cal_slope;
  double cal_offset;
  int i;

  if (argc < 2) {
    fprintf(stderr, "usage: %s <SATMOS XRIT file (with \"CYCLE\" in the name)>\n", argv[0]);
    exit(EXIT_FAILURE);
  }

  input_file = argv[1];
  read_xrit(input_file, NULL, &nsegments, &xrit_header, &rows_infos, &prologue, NULL);

  /* printf("nsegments = %d\n", nsegments); */
  /* print_xrit_header(xrit_header); */
  free_xrit_header(xrit_header);

  /*
  for (irow = 0 ; irow < XRIT_NROWS ; irow++) {
    
    if (rows_infos.validity[irow]) printf("%d %d %d %d %s", 1 + irow, rows_infos.validity[irow], rows_infos.radiometric_validity[irow],
				   rows_infos.geometric_validity[irow], asctime(gmtime(&rows_infos.time[irow])));
    
  }
  */

  printf("prologue.satelliteStatus.satelliteDefinition.satelliteID = %hd\n", ntohs(prologue.satelliteStatus.satelliteDefinition.satelliteID));
  printf("prologue.satelliteStatus.satelliteDefinition.nominalLongitude = %f\n", ntoh_float(prologue.satelliteStatus.satelliteDefinition.nominalLongitude));
  printf("prologue.satelliteStatus.satelliteDefinition.satelliteStatus = %d\n", prologue.satelliteStatus.satelliteDefinition.satelliteStatus);

  /*
  printf("offsetof(MSG_Prologue, satelliteStatus.satelliteDefinition.satelliteID) = %d\n", offsetof(MSG_Prologue, satelliteStatus.satelliteDefinition.satelliteID));
  printf("offsetof(MSG_Prologue, satelliteStatus.satelliteDefinition.nominalLongitude) = %d\n", offsetof(MSG_Prologue, satelliteStatus.satelliteDefinition.nominalLongitude));
  printf("offsetof(MSG_Prologue, satelliteStatus.satelliteDefinition.satelliteStatus) = %d\n", offsetof(MSG_Prologue, satelliteStatus.satelliteDefinition.satelliteStatus));
  */

  for (i = 0 ; i < 12 ; i++) {
    cal_slope = ntoh_double(prologue.radiometricProcessing.level1_5ImageCalibration[i].cal_Slope);
    cal_offset = ntoh_double(prologue.radiometricProcessing.level1_5ImageCalibration[i].cal_Offset);
  
    printf("prologue.radiometricProcessing.level1_5ImageCalibration[%d].cal_Slope = %lf\n", i, cal_slope);
    printf("prologue.radiometricProcessing.level1_5ImageCalibration[%d].cal_Offset = %lf\n", i, cal_offset);
  }

  exit(EXIT_SUCCESS);

}
